@@ -61,6 +61,15 @@
 #include "NetworkPreferencePage.hh"
 #endif
 
+#ifndef WR_CHECK_VERSION
+#define WR_CHECK_VERSION(comp,major,minor,micro)   \
+    (comp##_MAJOR_VERSION > (major) || \
+     (comp##_MAJOR_VERSION == (major) && comp##_MINOR_VERSION > (minor)) || \
+     (comp##_MAJOR_VERSION == (major) && comp##_MINOR_VERSION == (minor) && \
+      comp##_MICRO_VERSION >= (micro)))
+#endif
+
+
 // #include "PluginsPreferencePage.hh"
 
 #define RUNKEY "Software\\Microsoft\\Windows\\CurrentVersion\\Run"
@@ -280,25 +289,38 @@ PreferencesDialog::create_gui_page()
   panel->add_label(_("Language:"), languages_combo);
 #endif
 
+  bool show_autostart = false;
+
 #if defined(PLATFORM_OS_WIN32)
-  Gtk::Label *autostart_lab = Gtk::manage(GtkUtil::create_label(_("Start Workrave on Windows startup"), false));
-  autostart_cb = Gtk::manage(new Gtk::CheckButton());
-  autostart_cb->add(*autostart_lab);
-  autostart_cb->signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::on_autostart_toggled));
-
-  panel->add(*autostart_cb);
-
-  char value[MAX_PATH];
-  bool rc = Util::registry_get_value(RUNKEY, "Workrave", value);
-  autostart_cb->set_active(rc);
+  show_autostart = true;
+#elif defined(PLATFORM_OS_UNIX)
+  const char *desktop = g_getenv("XDG_CURRENT_DESKTOP");
+  show_autostart = (g_strcmp0(desktop, "Unity") == 0);
+#endif  
+  
+  if (show_autostart)
+    {
+      Gtk::Label *autostart_lab = Gtk::manage(GtkUtil::create_label(_("Start Workrave on logon"), false));
+      autostart_cb = Gtk::manage(new Gtk::CheckButton());
+      autostart_cb->add(*autostart_lab);
+      autostart_cb->signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::on_autostart_toggled));
+      panel->add_widget(*autostart_cb);
+      
+      connector->connect(GUIConfig::CFG_KEY_AUTOSTART, dc::wrap(autostart_cb));
+      
+#if defined(PLATFORM_OS_WIN32)
+      char value[MAX_PATH];
+      bool rc = Util::registry_get_value(RUNKEY, "Workrave", value);
+      autostart_cb->set_active(rc);
 #endif
+    }
 
   Gtk::Label *trayicon_lab = Gtk::manage(GtkUtil::create_label(_("Show system tray icon"), false));
   trayicon_cb = Gtk::manage(new Gtk::CheckButton());
   trayicon_cb->add(*trayicon_lab);
   connector->connect(GUIConfig::CFG_KEY_TRAYICON_ENABLED, dc::wrap(trayicon_cb));
 
-  panel->add_widget(*trayicon_cb);
+  panel->add_widget(*trayicon_cb, false, false);
   
   panel->set_border_width(12);
   return panel;
@@ -461,8 +483,13 @@ PreferencesDialog::create_sounds_page()
       selector_playbutton->signal_clicked().connect(sigc::mem_fun(*this,
                                                                   &PreferencesDialog::on_sound_filechooser_play));
 
+#if WR_CHECK_VERSION(GTKMM,2,22,0)
+      fsbutton->signal_file_set().connect(sigc::mem_fun(*this,
+                                                                 &PreferencesDialog::on_sound_filechooser_select));
+#else
       fsbutton->signal_selection_changed().connect(sigc::mem_fun(*this,
                                                                  &PreferencesDialog::on_sound_filechooser_select));
+#endif
 
       Glib::RefPtr<Gtk::TreeSelection> selection = sound_treeview.get_selection();
       selection->signal_changed().connect(sigc::mem_fun(*this,
@@ -745,23 +772,25 @@ PreferencesDialog::on_cell_data_compare(const Gtk::TreeModel::iterator& iter1,
 }
 #endif
 
-#if defined(PLATFORM_OS_WIN32)
 void
 PreferencesDialog::on_autostart_toggled()
 {
   bool on = autostart_cb->get_active();
+
+#if defined(PLATFORM_OS_WIN32)
   gchar *value = NULL;
 
   if (on)
     {
       string appdir = Util::get_application_directory();
-
       value = g_strdup_printf("%s" G_DIR_SEPARATOR_S "lib" G_DIR_SEPARATOR_S "workrave.exe", appdir.c_str());
     }
 
   Util::registry_set_value(RUNKEY, "Workrave", value);
+#endif
 }
 
+#if defined(PLATFORM_OS_WIN32)
 void
 PreferencesDialog::on_monitor_type_toggled()
 {
@@ -894,32 +923,35 @@ PreferencesDialog::on_sound_theme_changed()
   TRACE_ENTER("PreferencesDialog::on_sound_theme_changed");
   int idx = sound_theme_button->get_active_row_number();
 
-  SoundPlayer::Theme &theme = sound_themes[idx];
-
-  IGUI *gui = GUI::get_instance();
-  SoundPlayer *snd = gui->get_sound_player();
-  snd->activate_theme(theme);
-
-  Glib::RefPtr<Gtk::TreeSelection> selection = sound_treeview.get_selection();
-  Gtk::TreeModel::iterator iter = selection->get_selected();
-
-  if (iter)
+  if (idx >= 0 && idx < sound_themes.size())
     {
-      Gtk::TreeModel::Row row = *iter;
-      string event = (Glib::ustring) row[sound_model.label];
-      string filename;
+      SoundPlayer::Theme &theme = sound_themes[idx];
 
-      bool valid = CoreFactory::get_configurator()->get_value(string(SoundPlayer::CFG_KEY_SOUND_EVENTS) +
-                                                              row[sound_model.label],
-                                                              filename);
+      IGUI *gui = GUI::get_instance();
+      SoundPlayer *snd = gui->get_sound_player();
+      snd->activate_theme(theme);
 
-      if (valid && filename != "")
+      Glib::RefPtr<Gtk::TreeSelection> selection = sound_treeview.get_selection();
+      Gtk::TreeModel::iterator iter = selection->get_selected();
+
+      if (iter)
         {
-          TRACE_MSG(filename << " " <<row[sound_model.label]);
-          inhibit_events++;
-          fsbutton_filename = filename;
-          fsbutton->set_filename(filename);
-          inhibit_events--;
+          Gtk::TreeModel::Row row = *iter;
+          string event = (Glib::ustring) row[sound_model.label];
+          string filename;
+
+          bool valid = CoreFactory::get_configurator()->get_value(string(SoundPlayer::CFG_KEY_SOUND_EVENTS) +
+                                                                  row[sound_model.label],
+                                                                  filename);
+
+          if (valid && filename != "")
+            {
+              TRACE_MSG(filename << " " <<row[sound_model.label]);
+              inhibit_events++;
+              fsbutton_filename = filename;
+              fsbutton->set_filename(filename);
+              inhibit_events--;
+            }
         }
     }
   TRACE_EXIT();
